BeginPackage["ArmaData`","Misc`","Rarma`"]

ArmaData::usage = "ArmaData[a_, b_, sigu_, Nsam_] generates Nsam data points 
of an ARMA process with parameters a, b, sigu, the input being a Gaussian 
i.i.d. sequence. The initial conditions are automatically computed so as to 
make the data wide-sense stationary. "

Filter::usage = "Filter[a_, b_, input_, incon_] computes the response of the 
filter b(z)/a(z) to the input sequence input, starting at initial conditions
incon.  The filter realization is in state-space controller form, and its
dimension is max{p,q+1}. This must also be the dimension of incon."

CovSeq::usage = "CovSeq[data_, nr_] generates the sample covariances of the 
given data of orders 0 through nr."

Begin["`Private`"]

NormalDistribution/: Random[NormalDistribution[mu_:0, sigma_:1]] :=
        N[mu + sigma Sqrt[-2 Log[Random[]]] Cos[2Pi Random[]]]

Filter[a_List, b_List, input_List, incon_List] :=
   Module[{p,q,output,t,x},
           p = Length[a] - 1;
           q = Length[b] - 1;
           x = incon;
           output = Zeros[Length[input]];
           Do[(x = Drop[Prepend[x,input[[t]] - Take[a,-p] . Take[x,p]],-1];
               output[[t]] = Take[x,q+1] . b;),
              {t,Length[input]}];
           Return[output]]

InCon[a_List, sigu_, uinit_List] := 
   Module[{lam,r,v},
          r = Rarma[a,{1},sigu,Length[uinit]-1];
          {lam,v} = Eigensystem[Toeplitz[r,r]];
          Return[v . (sigu Sqrt[lam] uinit)]]

ArmaData[a_List, b_List, sigu2_, Nsam_Integer] :=
   Module[{dist,i,indat,init,p,q,sigu},
         dist = NormalDistribution[];
         p = Length[a] - 1;
         q = Length[b] - 1;
         sigu = N[Sqrt[sigu2]];
         init = InCon[a,sigu,Table[Random[dist],{Max[p,q+1]}]];
         indat = sigu Table[Random[dist],{Nsam}];
         Return[Filter[a,b,indat,init]]]

CovSeq[data_List, nr_Integer] :=
   Module[{i,ndat,r},
         ndat = Length[data];
         r = Zeros[nr+1];
         Do[r[[i+1]] = Take[data,ndat-i] . Take[data,-(ndat-i)],
            {i,0,nr}];
         Return[r/ndat]]

End[]
EndPackage[]
Null
