BeginPackage["CRarma`","Misc`","Rarma`"]

CRarma::usage = "CRarma[a_, b_, sigu2_, m_] computes the normalized covariance
matrix of the sample covariances of an ARMA process, of orders 0 through m."

CGarma::usage = "CGarma[a_, b_, m_] computes the normalized covariance
matrix of the sample correlation coefficients of an ARMA process, of orders
1 through m."

Begin["`Private`"]

CRarma[a_List, b_List, sigu2_, m_Integer] :=
  Module[{a2,b2,m2,sigu4,r},
        a2 = Convolution[a,a];
        b2 = Convolution[b,b];
        m2 = 2*m; sigu4 = sigu2^2;
        r = Rarma[a2,b2,sigu4,m2];
        Return[Toeplitz[Take[r,m+1],Take[r,m+1]] +
               Hankel[Take[r,m+1],Take[r,-(m+1)]]]]

CGarma[a_List, b_List, m_Integer] :=
  Module[{a2,b2,m2,r0,rho,r2,s11,s12,s22,temp},
        r0 = Rarma[a,b,1,0][[1]];
        rho = Garma[a,b,m];
        a2 = Convolution[a,a];
        b2 = Convolution[b,b];
        m2 = 2*m;
        r2 = Rarma[a2,b2,1,m2];
        s11 = 2 r2[[1]];
        s12 = 2 Take[r2,{2,m+1}];
        s22 = Toeplitz[Take[r2,m],Take[r2,m]] +
               Hankel[Take[r2,{3,m+2}],Take[r2,-m]];
        temp = Outer[Times,rho,s12];
        Return[r0^(-2)
              (s11 Outer[Times,rho,rho] + s22 - temp - Transpose[temp])]]

End[]
EndPackage[]
Null
