BeginPackage["CRarmad`","Misc`","Rarma`","Rarmad`","CRarma`"]

CRarmad::usage = "CRarmad[a_, b_, sigu2_, m_] computes the derivatives of the
normalized covariance matrix of the sample covariances of an ARMA process,
of orders 0 through m."

CGarmad::usage = "CGarmad[a_, b_, m_] computes the derivatives of the
normalized covariance matrix of the sample correlation coefficients of an ARMA
process, of orders 1 through m."

Begin["`Private`"]

CRarmad[a_List, b_List, sigu2_, m_Integer] :=
  Module[{i,p,q,a2,b2,m2,sigu4,temp,dr1,dr2,drdx,dsigma={}},
        p = Length[a] - 1;
        q = Length[b] - 1;
        a2 = Convolution[a,a];
        b2 = Convolution[b,b];
        m2 = 2*m; sigu4 = sigu2^2;
        dr1 = Rarmad[a2,b2,sigu4,m2];
        dr2 = {2 sigu2 dr1[[1]]};
        If[p > 0,
           (temp = Toeplitz[Prepend[Zeros[p-1],1],Join[a,Zeros[p-1]]];
            dr2 = Join[dr2, 2 temp . Take[dr1,{2,2*p+1}]];)];
        If[q > 0,
           (temp = Toeplitz[Prepend[Zeros[q-1],1],Join[b,Zeros[q-1]]];
            dr2 = Join[dr2, 2 temp . Take[dr1,{2*p+2,2*p+2*q+1}]];)];
        Do[(drdx = dr2[[i]]; AppendTo[dsigma,
            Toeplitz[Take[drdx,m+1],Take[drdx,m+1]] +
            Hankel[Take[drdx,m+1],Take[drdx,-(m+1)]]];),
           {i,1,p+q+1}];
        Return[dsigma]]

CGarmad[a_List, b_List, m_Integer] :=
  Module[{i,p,q,r0,dr0dx,rho,drhodx,sigma,dsigmadx,s11,s12,s22,ds11dx,ds12dx,
         ds22dx,dgdx = {},temp0,temp1,temp2,temp3},
        p = Length[a] - 1;
        q = Length[b] - 1;
        r0 = Rarma[a,b,1,0][[1]];
        rho = Garma[a,b,m];
        dr0dx = Flatten[Rest[Rarmad[a,b,1,0]]];
        drhodx = Garmad[a,b,m];
        sigma = CRarma[a,b,1,m];
        dsigmadx = Rest[CRarmad[a,b,1,m]];
        {s11,s12,s22} = Split[sigma];
        temp0 = Outer[Times,rho,rho];
        temp1 = Outer[Times,rho,s12];
        temp1 = -2 r0^(-3) (s11 temp0 + s22 - temp1 - Transpose[temp1]);
        Do[({ds11dx,ds12dx,ds22dx} = Split[dsigmadx[[i]]];
            temp2 = Outer[Times,drhodx[[i]],rho];
            temp2 = temp2 + Transpose[temp2];
            temp3 = Outer[Times,drhodx[[i]],s12];
            temp3 = temp3 + Transpose[temp3];
            temp2 = r0^(-2) (s11 temp2 - temp3);
            temp3 = Outer[Times,rho,ds12dx];
            temp3 = r0^(-2) (ds11dx temp0 + ds22dx - temp3 - Transpose[temp3]);
            AppendTo[dgdx,dr0dx[[i]] temp1 + temp2 + temp3];),
           {i,1,p+q}];
        Return[dgdx]]

Split[mat_] :=
   Module[{mat11,mat12,mat22},
         mat11 = First[First[mat]];
         mat12 = Rest[First[mat]];
         mat22 = Transpose[Rest[Transpose[Rest[mat]]]];
         Return[{mat11,mat12,mat22}]]

End[]
EndPackage[]
Null
