BeginPackage["Cumulant`","Liapunov`", "Misc`"]

Cum::usage = "Cum[{a_, b_, c_}, tau_, kappa_] computes the cumulant of the 
process generated by the state-space model {a, b, c}, with input cumulant 
function kappa, corresponding to  a list of indices tau. Cum[b_, tau_, kappa_]
is for a moving average process with coefficients b. Cum[{a_, b_}, tau_,
kappa_] is for an ARMA process with coefficients {a, b}."

kappaexp::usage = "kappaexp[n_] returns the n-th order cumulant of the 
shifted exponential distribution."

Begin["`Private`"]

Cum[{a_?MatrixQ, b_?MatrixQ, c_?MatrixQ}, tau:{__?IntegerQ}, kappa_] :=
   kappa[Length[tau]+1] H[{a,b,c},Sort[Append[tau,0] - Min[Append[tau,0]]]]

Cum[b:{__?NorSQ}, tau:{__?IntegerQ}, kappa_] :=
   kappa[Length[tau]+1] H[b,Sort[Append[tau,0] - Min[Append[tau,0]]]]

Cum[{a:{__?NumberQ}, b:{__?NumberQ}}, tau:{__?IntegerQ}, kappa_] :=
   Module[{aa,bb,cc,diag,i,j,n,temp},
          {aa,bb,cc} = Arma2SS[{a,b}];
          temp = Eigenvalues[aa];
          n = Length[temp];
          temp = Flatten[Table[Abs[temp[[i]]-temp[[j]]],{i,1,n},{j,i+1,n}]];
          diag = If[(n == 1) || (Max[temp] > 10.0^(-6)), True, False];
          {aa,bb,cc} = If[diag, SS2Diag[{aa,bb,cc}], SS2Schur[{aa,bb,cc}]];
          Return[Cum[{aa,bb,cc},tau,kappa]]]

kappaexp[0] = 1;
kappaexp[1] = 0;
kappaexp[n_Integer] := (n-1)! /; n > 1

End[]
EndPackage[]
Null
