BeginPackage["Gabor`", "Misc`"]

Bastiaans::usage = "Bastiaans[t_] computes the biorthogonal function of the 
Gaussian window."

GaborAnalysis::usage = "GaborAnalysis[data_, n_, m_, gamma_] performs 
discrete Gabor (i.e., short-time Fourier transform) analysis of the sequence 
data. n is the decimation ratio, m the number of frequencies, gamma the 
analysis window. Returns a two dimensional array, where each row is an array 
of frequency points for a fixed time."

GaborSynthesis::usage = "GaborSynthesis[data_, n_, m_, g_] performs 
discrete Gabor (i.e., short-time Fourier transform) synthesis of the two
dimensional array data. n is the decimation ratio, m the number of
frequencies, g the synthesis window. Returns the one dimensional time
sequence."
 
Begin["`Private`"]

Bastiaans[t_] :=
   Module[{n,n0,fac1,fac2},
          n0 = Ceiling[Abs[t] - 0.5];
          fac1 = NSum[(-1)^n E^(-Pi*((n+0.5)^2-t^2)), {n,n0,10}];
          fac2 = 2.0^(-0.25)*(1.854075/Pi)^(-1.5);
          Return[fac1 fac2]]

GaborAnalysis[data:{__?NumberQ}, n_?PosIntQ, m_?PosIntQ, gamma:{__?NumberQ}] :=
   Module[{i,l,q,register,res,sm,temp,w,window,winres,wm,y},
          l = Ceiling[Length[gamma]/m];
          q = l*m;
          window = ZeroPad[gamma,q];
          sm = N[Sqrt[m]];
          w = N[Exp[-I*2*Pi*n/m]^(Range[m]-1)];
          wm = Table[1.0,{m}];
          y = data;
          res = {};
          register = Join[Zeros[n],Take[y,q-n]];
          y = Drop[y,q-n];
          While[Length[y] > 0,
                (If[Length[y] >= n,
                    (register = Join[Drop[register,n],Take[y,n]];
                     y = Drop[y,n];),
                    (register = Join[Drop[register,n],ZeroPad[y,n]];
                     y = {};)];
                  winres = window*register;
                  temp = Zeros[m];
                  Do[temp = temp+Take[winres,{(i-1)*m+1,i*m}],{i,1,l}];
                  temp = sm*InverseFourier[temp]*wm;
                  wm = wm*w;
                  res = Append[res,temp];)];
          Return[res]]

GaborSynthesis[data_?MatrixQ, n_?PosIntQ, m_?PosIntQ, g:{__?NumberQ}] :=
   Module[{i,l,q,register,res,sm,temp,w,window,winres,wm,y},
          l = Ceiling[Length[g]/m];
          q = l*m;
          window = ZeroPad[g,q];
          sm = N[1/Sqrt[m]];
          w = N[Exp[I*2*Pi*n/m]^(Range[m]-1)];
          wm = Table[1.0,{m}];
          res = {};
          register = Zeros[q];
          Do[(temp = wm*data[[i]];
              wm = wm*w;
              temp = Chop[sm*Fourier[temp]];
              winres = g*Flatten[Table[temp,{l}]];
              register = register+winres;
              res = Join[res,Take[register,-n]];
              register = Join[Zeros[n],Drop[register,-n]];),
             {i,1,Length[data]}];
          Return[res]]

End[]
EndPackage[]
Null
