BeginPackage["NgData`","Misc`","Rarma`","MomCum`",
             "Statistics`ContinuousDistributions`"]

NgData::usage = "NgData[a_, b_, dist_, Nsam_] generates Nsam data points 
of an ARMA process with parameters a, b, the input being an i.i.d. sequence
from the supplied distribution dist. The initial conditions are automatically
computed so as to make the data wide-sense stationary (as in ArmaData)."

SampleMom::usage = "SampleMom[data_, stat_] generates the sample moments of
the given data specified by the list stat of desired indices."

SampleCum::usage = "SampleCum[data_, stat_, opts___] generates the sample
cumulants of the given data specified by the list stat of desired indices.
The optional parameter NoOnes->True should be used when the process is known
to have zero mean."

ShiftedExponential/: Random[ShiftedExponential[lambda_:1]] :=
        - Log[Random[]]/lambda -1/lambda // N

Begin["`Private`"]

Filter[a_List, b_List, input_List, incon_List] :=
   Module[{p,q,output,t,x},
           p = Length[a] - 1;
           q = Length[b] - 1;
           x = incon;
           output = Zeros[Length[input]];
           Do[(x = Drop[Prepend[x,input[[t]] - Take[a,-p] . Take[x,p]],-1];
               output[[t]] = Take[x,q+1] . b;),
              {t,Length[input]}];
           Return[output]]

NgData[a_List, b_List, dist_, Nsam_Integer] :=
   Module[{i,indat,init,p,q},
         p = Length[a] - 1;
         q = Length[b] - 1;
         init = Zeros[Max[p,q+1]];
         indat = Table[Random[dist],{Nsam}];
         Return[Filter[a,b,indat,init]]]

OneSampleMom[data_List, {}] := OneSampleMom[data,{}] =
   N[Apply[Plus,data]/Length[data]]

OneSampleMom[data_List, index_List] := OneSampleMom[data,index] =
   Module[{i,index1,Nsam,temp,tmax},
          index1 = Drop[Sort[Append[index,0]],1];
          tmax = Last[index1];
          Nsam = Length[data];
          temp = Take[data,Nsam-tmax];
          Do[temp = temp*Take[data,{index1[[i]]+1,Nsam+index1[[i]]-tmax}],
             {i,1,Length[index1]}];
          Return[N[Apply[Plus,temp]/Nsam]]]

SampleMom[data_List, stat_List] :=
   Module[{f,index},
          f[index_] := OneSampleMom[data,index];
          Return[Map[f,stat]]]

OneSampleCum[data_List, index_List, opts___] :=
    Module[{index1,mom},
           mom[index1_] :=
              Module[{indtmp},
                     indtmp = Sort[index1];
                     indtmp = Drop[indtmp,1] - indtmp[[1]];
                     Return[OneSampleMom[data,indtmp]]];
           Return[MomToCum[mom,Append[index,0],opts]]]

SampleCum[data_List, stat_List, opts___] :=
   Module[{f,index},
          f[index_] := OneSampleCum[data,index,opts];
          Return[Map[f,stat]]]

End[]
EndPackage[]
Null
