BeginPackage["Rarmad`","Misc`","Rarma`"]

Rarmad::usage = "Rarmad[a_, b_, sigu2_, m_]  computes the partial derivatives
of the covariances of an ARMA process of orders 0 through m with respect to 
the ARMA parameters."

Garmad::usage = "Garmad[a_, b_, m_]  computes the partial derivatives
of the correlation coefficients of an ARMA process of orders 1 through m with
respect to the ARMA parameters."

Begin["`Private`"]

Rarmada[a_List, b_List, sigu2_, m_Integer] :=
   Module[{a2,dr={},dr1,dr2,k,p,r},
         p = Length[a] - 1;
         a2 = Convolution[a,a];
         r = Rarma[a2,b,sigu2,m+p];
         r = Join[Reverse[Take[r,{2,p+1}]],r];
         dr1 = Take[Convolution[r,Reverse[a]],{p+1,m+3*p+1}];
         dr2 = Take[Convolution[r,a],{p+1,m+3*p+1}];
         Do[AppendTo[dr, -Take[dr1,{p-k+1,p-k+1+m}] - Take[dr2,{k+1,k+m+1}]],
            {k,1,p}];
         Return[dr]]

Rarmadb[a_List, b_List, sigu2_, m_Integer] :=
   Module[{k,dr={},dr1,dr2,q,r},
         q = Length[b] - 1;
         r = Rarma[a,{1},sigu2,m+q];
         r = Join[Reverse[Take[r,{2,q+1}]],r];
         dr1 = Take[Convolution[r,Reverse[b]],{q+1,m+3*q+1}];
         dr2 = Take[Convolution[r,b],{q+1,m+3*q+1}];
         Do[AppendTo[dr, Take[dr1,{q-k+1,q-k+1+m}] + Take[dr2,{k+1,k+m+1}]],
            {k,1,q}];
         Return[dr]]

Rarmad[a_List, b_List, sigu2_, m_Integer] :=
   Module[{mat,p,q},
         p = Length[a] - 1;
         q = Length[b] - 1;
         mat = {(1/sigu2) Rarma[a,b,sigu2,m]};
         If[p > 0, mat = Join[mat,Rarmada[a,b,sigu2,m]]];
         If[q > 0, mat = Join[mat,Rarmadb[a,b,sigu2,m]]];
         Return[mat]]

Garmad[a_List, b_List, m_Integer] :=
   Module[{dr,drfirst,drrest,r},
         r = Rarma[a,b,1,m];
         dr = Transpose[Rest[Rarmad[a,b,1,m]]];
         drfirst = dr[[1]];
         drrest = Transpose[Rest[dr]];
         Return[(r[[1]])^(-1) drrest -
                (r[[1]])^(-2) Outer[Times,drfirst,Rest[r]]]]

End[]
EndPackage[]
Null
