BeginPackage["SinData`","Misc`"]

SinData::usage = "SinData[a_, phi_, omega_, sigma_, Nsam_] generates a 
realization of multiple sinusoids in white Gaussian noise. a is the vector of 
amplitudes, phi the vector of phases, omega the vector of frequencies, sigma 
the noise standard deviation and Nsam the number of samples."

CovSeq::usage = "CovSeq[data_, nr_] generates the sample covariances of the 
given data of orders 0 through nr."

NAsymptotic::usage = "NAsymptotic[omega_] computes the minimum N such that
the approximate asymptotic analysis applies, according to formula (9.2.14)
in the book."

Begin["`Private`"]

NormalDistribution/: Random[NormalDistribution[mu_:0, sigma_:1]] :=
        N[mu + sigma Sqrt[-2 Log[Random[]]] Cos[2Pi Random[]]]

SinData[a_List, phi_List, omega_List, sigma_, Nsam_Integer] :=
   Module[{i,m,t},
          m = (Nsam-1)/2;
          Return[Table[Sum[a[[i]] Cos[omega[[i]] (t-m) - phi[[i]]] +
                 sigma Random[NormalDistribution[]],{i,1,Length[a]}],
                 {t,0,Nsam-1}]]]

CovSeq[data_List, nr_Integer] :=
   Module[{i,ndat,r},
          ndat = Length[data];
          r = Zeros[nr+1];
          Do[r[[i+1]] = Take[data,ndat-i] . Take[data,-(ndat-i)],
             {i,0,nr}];
          Return[r/ndat]]

NAsymptotic[omega_List] :=
   Module[{k,l,m,n1,n2,n3,n4,om,pi},
          pi = N[Pi];
          m = Length[omega];
          om = Mod[Abs[omega],pi];
          n1 = Min[Flatten[Table[Abs[om[[k]]-om[[l]]],{k,1,m},{l,k+1,m}]]];
          n2 = Min[Flatten[Table[2.0*pi-om[[k]]-om[[l]],{k,1,m},{l,k+1,m}]]];
          n3 = Min[2.0*om];
          n4 = Min[2.0*(pi-om)];
          Return[Ceiling[2.0*pi/Min[n1,n2,n3,n4]]]]



End[]
EndPackage[]
Null
