BeginPackage["Wavelet`", "Misc`"]

DaubechiesCqf::usage = "DaubechiesCqf[order_] computes the Daubechies
conjugate quadratic filter, as described in Section 11.6. Returns c, the list 
of impulse response coefficients."

DaubechiesWavelet::usage = "DaubechiesWavelet[order_, density_] computes
the Daubechies wavelet of a given order, at a given density of points per
second."

Begin["`Private`"]

DaubechiesCqf[order_Integer] :=
   Module[{c,n,p,r,z},
      p = z^(order-1) Sum[Binomial[order-1+n,n] ((2-z-z^(-1))/4)^n,
                          {n,0,order-1}];
      r = z - Select[Proots[N[CoefficientList[p,z]]],(Abs[#] < 1)&];
      p = Expand[Apply[Times,r] ((z+1)/2)^order];
      c = Chop[Reverse[CoefficientList[p,z]]];
      Return[(1/Apply[Plus,c]) c]]

DaubechiesWavelet[order_Integer, density_Integer] :=
   Module[{c,ce,co,even,fe,fo,h,he,ho,i,j,len,m,odd},
          c = 2 DaubechiesCqf[order];
          len = Length[c];
          odd = Range[1,len,2]; even = Range[2,len,2];
          co = c[[odd]]; ce = c[[even]];
          f = c; h = Reverse[c] (-1)^(Range[Length[c]]-1);
          m = 3 len - 2;
          Do[(fo = Convolution[co,f]; fe = Convolution[ce,f];
              ho = Convolution[co,h]; he = Convolution[ce,h];
              f = {}; h = {};
              Do[(f = Join[f,{fo[[j]],fe[[j]]}];
                  h = Join[h,{ho[[j]],he[[j]]}];), {j,1,m/2}];
              m = 2 m + len - 2;),
            {i,2,density}];
          f = Join[f,Zeros[len-2]]; h = Join[h,Zeros[len-2]];
          Return[{f,h}]]

End[]
EndPackage[]
Null
